/*
 *  File: IntervalImpl.java 
 *  Copyright (c) 2004-2005  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.util.date;

import de.jaret.util.misc.PropertyObservableBase;

/**
 * Simple Implementation of the <code>de.jaret.util.ui.timebars.model.Interval</code>.
 * @author Peter Kliem
 * @version $Id: IntervalImpl.java 243 2005-08-19 20:53:50Z kliem $
 */
public class IntervalImpl extends PropertyObservableBase implements Interval{
	protected JaretDate _begin;
	protected JaretDate _end;
	
	/* (non-Javadoc)
	 * @see de.kliemax.timebars.Interval#getBegin()
	 */
	public JaretDate getBegin() {
		return _begin;
	}

	/* (non-Javadoc)
	 * @see de.kliemax.timebars.Interval#getEnd()
	 */
	public JaretDate getEnd() {
		return _end;
	}
	/**
	 * @param begin The begin to set.
	 */
	public void setBegin(JaretDate begin) {
		JaretDate oldVal = _begin;
		_begin = begin;
		firePropertyChange("Begin", oldVal, begin);
	}
	/**
	 * @param end The end to set.
	 */
	public void setEnd(JaretDate end) {
		JaretDate oldVal = _end;
		_end = end;
		firePropertyChange("End", oldVal, end);
	}
	
	/* (non-Javadoc)
	 * @see de.kliemax.timebars.Interval#contains(de.kliemax.util.KliemaxDate)
	 */
	public boolean contains(JaretDate date) {
		return getBegin().compareTo(date)<=0 && getEnd().compareTo(date)>=0;
	}
	public static boolean containsNonIncluding(Interval interval, JaretDate date) {
		return interval.getBegin().compareTo(date)<0 && interval.getEnd().compareTo(date)>0;
	}
	/* (non-Javadoc)
	 * @see de.kliemax.timebars.Interval#getSeconds()
	 */
	public int getSeconds() {
		return getEnd().diffSeconds(getBegin());
	}
	
	public String toString() {
		return getBegin().toDisplayString()+"--"+getEnd().toDisplayString();
	}
	
	/* (non-Javadoc)
	 * @see de.kliemax.timebars.Interval#intersects(de.kliemax.timebars.Interval)
	 */
	public boolean intersects(Interval interval) {
		return intersect(this, interval);
	}
	public static boolean intersect(Interval i1, Interval i2) {
		if (i1.contains(i2.getBegin()) || i1.contains(i2.getEnd())) {
			return true;
		}
		if (i2.contains(i1.getBegin())||i2.contains(i1.getEnd())) {
			return true;
		}
		return false;
	}
	public static boolean intersectNonIncluding(Interval i1, Interval i2) {
		if (containsNonIncluding(i1, i2.getBegin()) || containsNonIncluding(i1, i2.getEnd())) {
			return true;
		}
		if (containsNonIncluding(i2, i1.getBegin())||containsNonIncluding(i2, i1.getEnd())) {
			return true;
		}
		return false;
	}
}
