/*
 *  File: GraphicsHelper.java 
 *  Copyright (c) 2004-2005  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
 package de.jaret.util.swing;

import java.awt.Graphics;
import java.awt.geom.Rectangle2D;

/**
 * A simple class containing several static methods for convenient painting with a grpahics object.
 * @author Peter Kliem
 * @version $Id: GraphicsHelper.java 125 2005-06-28 10:40:03Z kliem $
 */
public class GraphicsHelper {

	
	public static void drawStringCentered(Graphics graphics, String string, int left, int right, int y) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		int width = right -left;
		int xx = (int) ((width - rect.getWidth())/2);
		graphics.drawString(string, xx, y);
		
	}

	public static void drawStringCenteredVCenter(Graphics graphics, String string, int left, int right, int yCenter) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		int descent = graphics.getFontMetrics().getDescent();
		int width = right -left;
		int xx = (int) ((width - rect.getWidth())/2);
		int y = yCenter + (int)(rect.getHeight()/2-descent);
		graphics.drawString(string, xx, y);
		
	}

	public static void drawStringRightAlignedVCenter(Graphics graphics, String string, int x, int y) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		
		int xx = (int) (x- rect.getWidth());
		int yy = (int) (y + (rect.getHeight() / 2));
		graphics.drawString(string, xx, yy);
	}
	public static void drawStringLeftAlignedVCenter(Graphics graphics, String string, int x, int y) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		
		int xx = x;
		int yy = (int) (y + (rect.getHeight() / 2));
		graphics.drawString(string, xx, yy);
	}
	
	/**
	 * @param graphics graphics to use
	 * @param string string to render
	 * @param x center x
	 * @param y basey
	 */
	public static void drawStringCentered(Graphics graphics, String string, int x, int y) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		int xx = x-(int) ((rect.getWidth())/2);
		graphics.drawString(string, xx, y);
	}

	public static int getStringDrawingWidth(Graphics graphics, String string) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		return (int)rect.getWidth();
	}
	public static int getStringDrawingHeight(Graphics graphics, String string) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		return (int)rect.getHeight()-graphics.getFontMetrics().getDescent();
	}

	public static void drawStringRightAlignedVTop(Graphics graphics, String string, int x, int yTop) {
		Rectangle2D rect = graphics.getFontMetrics().getStringBounds(string, graphics);
		
		int xx = (int) (x- rect.getWidth());
		int yy = (int) (yTop + rect.getHeight());
		graphics.drawString(string, xx, yy);
		
	}

	public static void drawArrowLine(Graphics g, int x1, int y1, int x2, int y2, int dist, boolean arrowLeft, boolean arrowRight) {
		int off = 3;
		g.drawLine(x1+off+1,y1,x2-off-1,y2);
		if (arrowLeft) {
			g.drawLine(x1, y1, x1+dist, y1-off);
			g.drawLine(x1, y1, x1+dist, y1+off);
			g.drawLine(x1+dist, y1-off, x1+dist, y1+off);
		}
		if (arrowRight) {
			g.drawLine(x2, y2, x2-dist, y2-off);
			g.drawLine(x2, y2, x2-dist, y2+off);
			g.drawLine(x2-dist, y2-off, x2-dist, y2+off);
		}
	}
	
	
}
