/*
 *  File: SwtGraphicsHelper.java 
 *  Copyright (c) 2004-2005  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.util.swt;


import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

/**
 * A simple class containing several static methods for convenient painting with a SWT gc.
 * @author Peter Kliem
 * @version $Id: SwtGraphicsHelper.java 219 2005-07-31 17:04:25Z kliem $
 */
public class SwtGraphicsHelper {

	public static void drawStringCentered(GC gc, String string, int left, int right, int y) {
		Point extent = gc.textExtent(string);
		int width = right -left;
		int xx = (int) ((width - extent.x)/2);
		gc.drawString(string, xx, y);
		
	}

	public static void drawStringCenteredVCenter(GC gc, String string, int left, int right, int yCenter) {
		Point extent = gc.textExtent(string);
	//	int descent = graphics.getFontMetrics().getDescent();
		int descent = 0;
		int width = right -left;
		int xx = (int) ((width - extent.x)/2);
		int y = yCenter - (int)(extent.y/2-descent);
		gc.drawString(string, left+xx, y);
		
	}

	public static void drawStringRightAlignedVCenter(GC gc, String string, int x, int y) {
		Point extent = gc.textExtent(string);
		
		int xx = (int) (x- extent.x);
		int yy = (int) (y - (extent.y / 2));
		gc.drawString(string, xx, yy);
	}
	public static void drawStringLeftAlignedVCenter(GC gc, String string, int x, int y) {
		Point extent = gc.textExtent(string);
		
		int xx = x;
		int yy = (int) (y + (extent.y / 2));
		gc.drawString(string, xx, yy);
	}
	
	public static void drawStringCentered(GC gc, String string, int xCenter, int yBase) {
		Point extent = gc.textExtent(string);
		int xx = xCenter-(int) ((extent.x)/2);
		gc.drawText(string, xx, yBase-extent.y, true);
	}

	/**
	 * Draw the string centered in the given rectangle (specified by plain values)
	 * @param gc
	 * @param string
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 */
	public static void drawStringCentered(GC gc, String string, int x, int y, int width, int height) {
		Point extent = gc.textExtent(string);
		int xx = x+ (width-extent.x)/2;
		int yy = y+ (height-extent.y)/2;
		gc.drawText(string, xx, yy, true);
	}

	/**
	 * Draw String centered in the given rectangle.
	 * @param gc
	 * @param string
	 * @param rect
	 */
	public static void drawStringCentered(GC gc, String string, Rectangle rect) {
		drawStringCentered(gc, string, rect.x, rect.y, rect.width, rect.height);
	}
	
	public static int getStringDrawingWidth(GC gc, String string) {
		return gc.textExtent(string).x;
	}
	public static int getStringDrawingHeight(GC gc, String string) {
		return gc.textExtent(string).y;
	}

	public static void drawStringRightAlignedVTop(GC gc, String string, int x, int yTop) {
		Point extent = gc.textExtent(string);
		
		int xx = (int) (x- extent.x);
		gc.drawText(string, xx, yTop, true);
		
	}

	public static void drawArrowLine(GC gc, int x1, int y1, int x2, int y2, int dist, boolean arrowLeft, boolean arrowRight) {
		int off = 3;
		gc.drawLine(x1+off+1,y1,x2-off-1,y2);
		if (arrowLeft) {
			gc.drawLine(x1, y1, x1+dist, y1-off);
			gc.drawLine(x1, y1, x1+dist, y1+off);
			gc.drawLine(x1+dist, y1-off, x1+dist, y1+off);
		}
		if (arrowRight) {
			gc.drawLine(x2, y2, x2-dist, y2-off);
			gc.drawLine(x2, y2, x2-dist, y2+off);
			gc.drawLine(x2-dist, y2-off, x2-dist, y2+off);
		}
	}
	
	
}
